import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import * as jwt from 'jsonwebtoken';

import { FactionEntity } from '../faction/faction.entity';
import { GameEntity } from '../game/game.entity';
import { TrackingService } from '../tracking/tracking.service';
import { UserService } from '../user/user.service';
import { FactionService } from '../faction/faction.service';
import { TrackingEntity } from '../tracking/tracking.entity';
import {
  MapDrawingEntity,
  MapDrawingHistoryEntity,
} from '../draw/coordinate.entity';

@Injectable()
export class ReplayService {
  constructor(
    @InjectRepository(FactionEntity)
    private factionRepository: Repository<FactionEntity>,
    @InjectRepository(GameEntity)
    private gameRepository: Repository<GameEntity>,
    @InjectRepository(TrackingEntity)
    private trackingRepository: Repository<TrackingEntity>,
    @InjectRepository(MapDrawingEntity)
    private mapdrawingRepository: Repository<MapDrawingEntity>,
    @InjectRepository(MapDrawingHistoryEntity)
    private mapHistoryRepository: Repository<MapDrawingHistoryEntity>,
    private trackingService: TrackingService,
    private userService: UserService,
    private factionService: FactionService,
  ) {}

  async replayData(gameId) {
    let mapDrawingIds = await this.mapdrawingRepository.find({
      where: { gameId: gameId },
      select: ['mapDrawingId'],
    });
    let drawings = [];
    await Promise.all(
      mapDrawingIds.map(async mapId => {
        drawings.push(
          await this.mapHistoryRepository.find({
            mapdrawing: mapId.mapDrawingId,
          }),
        );
      }),
    );
    return drawings;
  }

  // get replay data for players
  async getPlayers(gameId) {
    let playerdata = await this.trackingRepository.find({
      where: { game: gameId },
      relations: ['faction', 'gamepersonId'],
    });

    // parse data
    const currentdata = await Promise.all(
      playerdata.map(async player => {
        player['data'][0]['info'] = [
          { key: 'icon', value: player.icon },
          { key: 'colour', value: player.faction.colour },
        ];
        return player['data'];
      }),
    );

    return currentdata;
  }
  // generate mockdata for a 3 day game
  async mockdata(gameId) {
    const users = [];
    const gamepersons = [];
    const game = await this.gameRepository.findOne({
      where: { id: gameId },
      relations: ['factions'],
    });
    const groups = await this.factionService.showGroups(
      game.factions[0].factionId,
    );
    for (let i = 0; i < 100; i++) {
      let res = await this.userService.register({
        name: 'asdf' + i,
        password: 'asd',
      });
      let user = await jwt.verify(res.token, process.env.SECRET);
      users.push(user);
      /*       let gameperson = await this.factionService.joinFaction(user['id'], {
        factionId:
          i < 10 ? game.factions[0].factionId : game.factions[1].factionId,
        factionPassword:
          i < 10
            ? game.factions[0].factionPassword
            : game.factions[1].factionPassword,
        game: gameId,
      }); */
      let gameperson = await this.factionService.joinFaction(user['id'], {
        factionId: game.factions[0].factionId,
        factionPassword: game.factions[0].factionPassword,
        game: gameId,
      });
      await this.factionService.joinGroup(gameperson, {
        groupId: groups[Math.floor(Math.random() * 3)],
      });
      gamepersons.push(gameperson);
    }
    let date: number = Date.now();
    let y1 = 25.7;
    let x1 = 62.3;
    let y2 = 25.75;
    let x2 = 62.35;
    for (let i = 1; i < 6; i++) {
      let x = 0;
      date += 10000;
      await Promise.all(
        gamepersons.map(async gameperson => {
          x++;
          let dataObject = {
            lat:
              x < 10
                ? x1 + ((i + Math.random()) * 5) / 2000
                : x2 - ((i + Math.random()) * 5) / 2000,
            lng:
              x < 10
                ? y1 + ((i + Math.random()) * 5) / 2000
                : y2 - ((i + Math.random()) * 5) / 2000,
            time: date,
          };
          await this.trackingService.trackLocation(
            gameperson,
            gameId,
            dataObject,
          );
        }),
      );
    }
    return {
      message: 'all done',
    };
  }
}
